@@ -1,6 +1,8 @@
 use ach_ring::Ring;
+use core::sync::atomic::Ordering::Relaxed;
 use std::ops::Range;
 use std::thread;
+use util::{MemoryRing, MemoryState};
 
 const TEST_DATA: Range<usize> = 0..1000;
 
@@ -22,7 +24,7 @@ fn test() {
     for _ in TEST_DATA {
         handle.push(thread::spawn(move || loop {
             let result = ARRAY.pop();
-            if let Some(_) = result {
+            if let Ok(_) = result {
                 break;
             } else {
                 thread::yield_now();
@@ -32,5 +34,9 @@ fn test() {
     for h in handle {
         let _ = h.join();
     }
+    assert!(ARRAY
+        .ops
+        .iter()
+        .all(|x| { x.load(Relaxed) == MemoryRing::new(10, MemoryState::Uninitialized) }));
     assert!(ARRAY.is_empty());
 }
